#include "includes.h"



/* ---------------------- HD44780 COMMANDS -------------------- */
#define  DISP_CMD_CLS            0x01  /* Clr display : clears display and returns cursor home         */

#if      DISP_BUS_WIDTH == 4
  #define  DISP_CMD_FNCT           0x28  /* Function Set: Set 4 bit data length, 1/16 duty, 5x8 dots     */
#else
  #define  DISP_CMD_FNCT           0x38  /* Function Set: Set 8 bit data length, 1/16 duty, 5x8 dots     */
#endif

#define  DISP_CMD_FNCT_INIT8     0x30  /* Function Set: 8 bit mode, no options                         */
#define  DISP_CMD_FNCT_INIT4     0x20  /* Function Set: 4 bit mode, no options                         */

#define  DISP_CMD_MODE           0x06  /* Entry mode  : Inc. display data address when writing         */
#define  DISP_CMD_ON_OFF         0x0C  /* Disp ON/OFF : Display ON, cursor OFF and no BLINK character  */

/*
*********************************************************************************************************
*                                            LOCAL VARIABLES
*********************************************************************************************************
*/

static  INT8U       DispMaxCols;       /* Maximum number of columns (i.e. characters per line)         */
static  INT8U       DispMaxRows;       /* Maximum number of rows for the display                       */


static  INT8U       DispBar1[] = {0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x80};
static  INT8U       DispBar2[] = {0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x80};
static  INT8U       DispBar3[] = {0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x80};
static  INT8U       DispBar4[] = {0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x80};
static  INT8U       DispBar5[] = {0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x80};

/*
*********************************************************************************************************
*                                        LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void        DispCursorSet(INT8U row, INT8U col);

#define DISP_RW_LOW   _PTBD.Bits.PTBD3 = 0
#define DISP_RW_HIGH  _PTBD.Bits.PTBD3 = 1
#define DISP_E_LOW    _PTBD.Bits.PTBD2 = 0
#define DISP_E_HIGH   _PTBD.Bits.PTBD2 = 1
#define DISP_S_LOW    _PTBD.Bits.PTBD1 = 0
#define DISP_S_HIGH   _PTBD.Bits.PTBD1 = 1
#define DISP_WR_HIGH_NIBLE(data) {PTBD = (PTBD & 0x0F) | (data & 0xF0);}
#define DISP_WR_LOW_NIBLE(data)  {PTBD = (PTBD & 0x0F) | (data << 4);}


void DispLock(void)
{
}
void DispUnlock(void)
{
}

/*
********************************************************************************************************
                                      WRITE DATA TO DISPLAY DEVICE

 Description : This function sends a single BYTE to the display device.
 Arguments   : 'data'  is the BYTE to send to the display device
 Returns     : none
 Notes       : 1) The board uses a 4 bit interface.
                  If an 8 bit interface is used  DispDataWr() will need
                  to be modified to reflect the new databus. In 8 bit mode, DispDataWrOneNibble()
                  is not necessary.
********************************************************************************************************
*/

void  DispDataWr (INT8U data)
{
  DISP_RW_LOW;                           /* Set R/W write LOW to write to the LCD module            */
  DISP_E_HIGH;                           /* Write the UPPER nibble to the LCD module                */
  DISP_WR_HIGH_NIBLE(data);
  Timer1_delay(T_TO_T1CYCLE(120e-6));
  DISP_E_LOW;

  Timer1_delay(T_TO_T1CYCLE(50e-6));                        /* Write the LOWER nibble to the LCD module                */
  DISP_E_HIGH;
  DISP_WR_LOW_NIBLE(data);
  Timer1_delay(T_TO_T1CYCLE(120e-6));
  DISP_E_LOW;
}

void  DispDataWrOneNibble(INT8U data)
{
  DISP_RW_LOW;                            /* Set R/W write LOW to write to the LCD module            */
  DISP_E_HIGH;                           /* Write the UPPER nibble to the LCD module                */
  DISP_WR_HIGH_NIBLE(data);
  Timer1_delay(T_TO_T1CYCLE(120e-6));
  DISP_E_LOW;
}



/*
********************************************************************************************************
                                   SELECT COMMAND OR DATA REGISTER

 Description : This changes the Register Select control line to the LCD controller.
 Arguments   : none
********************************************************************************************************
*/

void  DispSel (INT8U sel)
{
  if ( sel == DISP_SEL_CMD_REG )
  {
    DISP_S_LOW;           /* Select the command register (RS low)                         */
  }
  else
  {
    DISP_S_HIGH;           /* Select the data    register (RS high)                        */
  }
}

/*
*********************************************************************************************************
*                                           DISPLAY A CHARACTER
*
* Description : This function is used to display a single character on the display device
* Arguments   : 'row'   is the row    position of the cursor in the LCD Display
*                       'row' can be a value from 0 to 'DispMaxRows - 1'
*               'col'   is the column position of the cursor in the LCD Display
*                       'col' can be a value from 0 to 'DispMaxCols - 1'
*               'c'     is the character to write to the display at the current ROW/COLUMN position.
* Returns     : none
*********************************************************************************************************
*/

void  DispChar (INT8U row, INT8U col, INT8U c)
{
  if ( row < DispMaxRows && col < DispMaxCols )
  {
    DispLock();
    DispCursorSet(row, col);            /* Position cursor at ROW/COL                              */
    DispSel(DISP_SEL_DATA_REG);
    DispDataWr(c);                      /* Send character to display                               */
    DispUnlock();
  }
}


void DispCursorOff(void)
{
  DispLock();
  DispSel(DISP_SEL_CMD_REG);
  DispDataWr(0x0C);
  DispUnlock();
}

void DispCursorOn(void)
{
  DispLock();
  DispSel(DISP_SEL_CMD_REG);
  DispDataWr(0x0E);
  DispUnlock();
}

void DispCursorBlink(void)
{
  DispLock();
  DispSel(DISP_SEL_CMD_REG);
  DispDataWr(0x0F);
  DispUnlock();
}

void DispSetCursorPos(INT8U row, INT8U col)
{
  DispLock();
  DispCursorSet (row,col);
  DispUnlock();
}
/*
*********************************************************************************************************
*                                             CLEAR LINE
*
* Description : This function clears one line on the LCD display and positions the cursor at the
*               beginning of the line.
* Arguments   : 'line'  is the line number to clear and can take the value
*                       0 to 'DispMaxRows - 1'
* Returns     : none
*********************************************************************************************************
*/

void  LCD_ClrLine (INT8U line)
{
  INT8U i;


  if ( line < DispMaxRows )
  {
    DispLock();
    DispCursorSet(line, 0);                  /* Position cursor at begin of the line to clear      */
    DispSel(DISP_SEL_DATA_REG);              /* Select the LCD Display DATA register               */
    for ( i = 0; i < DispMaxCols; i++ )
    {      /* Write ' ' into all column positions of that line   */
      DispDataWr(' ');                     /* Write an ASCII space at current cursor position    */
    }
    DispCursorSet(line, 0);                  /* Position cursor at begin of the line to clear      */
    DispUnlock();
  }
}


/*
*********************************************************************************************************
*                                            CLEAR THE SCREEN
*
* Description : This function clears the display
* Arguments   : none
* Returns     : none
*********************************************************************************************************
*/

void  LCD_ClrScr (void)
{
  DispLock();
  DispSel(DISP_SEL_CMD_REG);         /* Select the LCD display command register                      */
  DispDataWr(DISP_CMD_CLS);          /* Send command to LCD display to clear the display             */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Delay at least  2 mS                                         */
  DispUnlock();
}


/*
*********************************************************************************************************
*                                     POSITION THE CURSOR (Internal)
*
* Description : This function positions the cursor into the LCD buffer
* Arguments   : 'row'   is the row    position of the cursor in the LCD Display
*                       'row' can be a value from 0 to 'DispMaxRows - 1'
*               'col'   is the column position of the cursor in the LCD Display
*                       'col' can be a value from 0 to 'DispMaxCols - 1'
* Returns     : none
*********************************************************************************************************
*/

static  void  DispCursorSet (INT8U row, INT8U col)
{
  DispSel(DISP_SEL_CMD_REG);                             /* Select LCD display command register      */
  switch ( row )
  {
    case 0:
      if ( DispMaxRows == 1 )
      {                       /* Handle special case when only one line   */
        if ( col < (DispMaxCols >> 1) )
        {
          DispDataWr(0x80 + col);               /* First  half of the line starts at 0x80   */
        }
        else
        {                                  /* Second half of the line starts at 0xC0   */
          DispDataWr(0xC0 + col - (DispMaxCols >> 1));
        }
      }
      else
      {
        DispDataWr(0x80 + col);                   /* Select LCD's display line 1              */
      }
      break;

    case 1:
      DispDataWr(0xC0 + col);                       /* Select LCD's display line 2              */
      break;

    case 2:
      DispDataWr(0x80 + DispMaxCols + col);         /* Select LCD's display line 3              */
      break;

    case 3:
      DispDataWr(0xC0 + DispMaxCols + col);         /* Select LCD's display line 4              */
      break;
  }
}


/*
*********************************************************************************************************
*                                            DEFINE CHARACTER
*
* Description : This function defines the dot pattern for a character.
* Arguments   : 'id'    is the identifier for the desired dot pattern.
*               'pat'   is a pointer to an 8 BYTE array containing the dot pattern.
* Returns     : None.
*********************************************************************************************************
*/

void  DispDefChar (INT8U id, INT8U *pat)
{
  INT8U i;


  DispLock();
  DispSel(DISP_SEL_CMD_REG);              /* Select command register                                 */
  DispDataWr(0x40 + (id << 3));           /* Set address of CG RAM                                   */
  DispSel(DISP_SEL_DATA_REG);             /* Select the data register                                */
  for ( i = 0; i < 8; i++ )
  {
    DispDataWr(*pat++);                 /* Write pattern into CG RAM                               */
  }
  DispUnlock();
}


/*
*********************************************************************************************************
*                                            DUMMY FUNCTION
*
* Description : This function doesn't do anything.  It is used to act like a NOP (i.e. No Operation) to
*               waste a few CPU cycles and thus, act as a short Delay.
* Arguments   : none
* Returns     : none
*********************************************************************************************************
*/

void  DispDummy (void)
{
}


/*
*********************************************************************************************************
*                                       DISPLAY A HORIZONTAL BAR
*
* Description : This function allows you to display horizontal bars (bar graphs) on the LCD module.
* Arguments   : 'row'   is the row    position of the cursor in the LCD Display
*                       'row' can be a value from 0 to 'DispMaxRows - 1'
*               'val'   is the value of the horizontal bar.  This value cannot exceed:
*                           DispMaxCols * 5
* Returns     : none
* Notes       : To use this function, you must first call DispHorBarInit()
*********************************************************************************************************
*/

void  DispHorBar (INT8U row, INT8U col, INT8U val)
{
  INT8U i;
  INT8U full;
  INT8U fract;


  full  = val / 5;                        /* Find out how many 'full' blocks to turn ON              */
  fract = val % 5;                        /* Compute portion of block                                */
  if ( row < DispMaxRows && (col + full - 1) < DispMaxCols )
  {
    DispLock();
    i = 0;                              /* Set counter to limit column to maximum allowable column */
    DispCursorSet(row, col);            /* Position cursor at beginning of the bar graph           */
    DispSel(DISP_SEL_DATA_REG);
    while ( full > 0 )
    {                  /* Write all 'full' blocks                                 */
      DispDataWr(5);                  /* Send custom character #5 which is full block            */
      i++;                            /* Increment limit counter                                 */
      full--;
    }
    if ( fract > 0 )
    {
      DispDataWr(fract);              /* Send custom character # 'fract' (i.e. portion of block) */
    }
    DispUnlock();
  }
}


const INT8U ScaleSym_patterns[8][8]={{0x80,0x80,0x80,0x80,0x80,0x80,0xff,0x80},
  {0x80,0x80,0x80,0x80,0x80,0xff,0xff,0x80},
  {0x80,0x80,0x80,0x80,0xff,0xff,0xff,0x80},
  {0x80,0x80,0xff,0xff,0xff,0xff,0xff,0x80},
  {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x80},
  {0x88,0x94,0x88,0x86,0x89,0x88,0x89,0x86},
  {0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80},
  {0x80,0x80,0x80,0x80,0x80,0x80,0x84,0x80}};

void DispChangeSym6Pattern(INT8U n)
{
  switch (n)
  {
    case 1:
      DispDefChar(6, (INT8U*)DispBar1);
      break;
    case 2:
      DispDefChar(6, (INT8U*)DispBar2);
      break;
    case 3:
      DispDefChar(6, (INT8U*)DispBar3);
      break;
    case 4:
      DispDefChar(6, (INT8U*)DispBar4);
      break;
    case 5:
      DispDefChar(6, (INT8U*)DispBar5);
      break;
    default:
      DispDefChar(6, (INT8U*)ScaleSym_patterns[6]);
      break;

  }

}

void DispLoadScaleSymPatterns(void)
{
  char i;
  for ( i=0;i<8;i++) DispDefChar(i, (INT8U*)ScaleSym_patterns[i]);
}


/*
*********************************************************************************************************
*                                      INITIALIZE HORIZONTAL BAR
*
* Description : This function is used to initialize the bar graph capability of this module.  You must
*               call this function prior to calling DispHorBar().
* Arguments   : none
* Returns     : none
*********************************************************************************************************
*/

void  DispHorBarInit (void)
{
  DispDefChar(1, &DispBar1[0]);
  DispDefChar(2, &DispBar2[0]);
  DispDefChar(3, &DispBar3[0]);
  DispDefChar(4, &DispBar4[0]);
  DispDefChar(5, &DispBar5[0]);
}


/*
*********************************************************************************************************
*                                      DISPLAY DRIVER INITIALIZATION
*
* Description : This function initializes the display driver.
* Arguments   : maxrows      specifies the number of lines on the display (1 to 4)
*               maxcols      specified the number of characters per line
* Returns     : None.
* Notes       : - DispInit() MUST be called only when multitasking has started.  This is because
*                 DispInit() requires time Delay services from the operating system.
*               - DispInit() MUST only be called once during initialization.
*********************************************************************************************************
*/

void  LCD_Init (INT8U maxrows, INT8U maxcols)
{

  DispMaxRows = maxrows;
  DispMaxCols = maxcols;

  /* INITIALIZE THE DISPLAY MODULE                                */
  DispSel(DISP_SEL_CMD_REG);         /* Select command register.                                     */
  Timer1_delay(T_TO_T1CYCLE(50000e-6));                 /* Delay more than 15 mS after power up                         */

#if DISP_BUS_WIDTH == 4
  DispDataWrOneNibble(DISP_CMD_FNCT_INIT8);/* Function Set: 8 bit, Only writes upper nibble          */
  Timer1_delay(T_TO_T1CYCLE(5000e-6));                  /* Busy flag cannot be checked yet!  5 FNCT writes recommended! */

  DispDataWrOneNibble(DISP_CMD_FNCT_INIT8);/* Function Set: 8 bit, Only writes upper nibble          */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Busy flag cannot be checked yet!                             */

  DispDataWrOneNibble(DISP_CMD_FNCT_INIT8);/* Function Set: 8 bit, Only writes upper nibble          */
  Timer1_delay(T_TO_T1CYCLE(5000e-6));                  /* Busy flag cannot be checked yet!                             */

  DispDataWrOneNibble(DISP_CMD_FNCT_INIT4);/* Function Set: 4 bit, Only writes upper nibble */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Busy flag cannot be checked yet!                             */
#else
  DispDataWr(DISP_CMD_FNCT_INIT8);    /* Two lines, 1/16 duty cycle, 5x8 dots, 8 bit operation         */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* 4 FNCT writes recommended in Hitachi datasheet!               */

  DispDataWr(DISP_CMD_FNCT_INIT8);    /* Two lines, 1/16 duty cycle, 5x8 dots, 8 bit operation         */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Busy flag cannot be checked yet!                              */

  DispDataWr(DISP_CMD_FNCT_INIT8);    /* Two lines, 1/16 duty cycle, 5x8 dots, 8 bit operation         */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Busy flag cannot be checked yet!                              */
#endif

  DispDataWr(DISP_CMD_FNCT);         /* Two lines, 1/16 duty cycle, 5x8 dots, Operation Mode         */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Optional: Busy flag can now be checked yet                   */

  DispDataWr(DISP_CMD_ON_OFF);       /* Disp ON/OFF: Display ON, cursor OFF and no BLINK character   */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Delay at least  2 mS                                         */

  DispDataWr(DISP_CMD_CLS);          /* Send command to LCD display to clear the display             */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Delay at least  2 mS                                         */

  DispDataWr(DISP_CMD_MODE);         /* Entry mode: Inc. display data address when writing           */
  Timer1_delay(T_TO_T1CYCLE(4000e-6));                  /* Delay at least  2 mS                                         */
}


/*
*********************************************************************************************************
*                                         DISPLAY AN ASCII STRING
*
* Description : This function is used to display an ASCII string on a line of the LCD display
* Arguments   : 'row'   is the row    position of the cursor in the LCD Display
*                       'row' can be a value from 0 to 'DispMaxRows - 1'
*               'col'   is the column position of the cursor in the LCD Display
*                       'col' can be a value from 0 to 'DispMaxCols - 1'
*               's'     is a pointer to the string to write to the display at
*                       the desired row/col.
* Returns     : none
*********************************************************************************************************
*/

void  LCD_DispStr (INT8U row, INT8U col, INT8U *s)
{
  INT8U i;


  if ( row < DispMaxRows && col < DispMaxCols )
  {
    DispLock();
    DispCursorSet(row, col);            /* Position cursor at ROW/COL                              */
    DispSel(DISP_SEL_DATA_REG);
    i = col;                            /* Set counter to limit column to maximum allowable column */
    while ( i < DispMaxCols && *s )
    {     /* Write all chars within str + limit to DispMaxCols       */
      DispDataWr(*s++);               /* Send character to LCD display                           */
      i++;                            /* Increment limit counter                                 */
    }
    DispUnlock();
  }
}

void  DispBlk (INT8U row, INT8U col, INT8U n, INT8U *s)
{
  INT8U i;


  if ( row < DispMaxRows && col < DispMaxCols )
  {
    DispLock();
    DispCursorSet(row, col);            /* Position cursor at ROW/COL                              */
    DispSel(DISP_SEL_DATA_REG);
    i = col;                            /* Set counter to limit column to maximum allowable column */
    while ( i < DispMaxCols && n!=0)
    {
      DispDataWr(*s++);               /* Send character to LCD display                           */
      i++;
      n--;
    }
    DispUnlock();
  }
}

