#include "includes.h"


/*===============================================================================================================
                                        SPI1C1 Field Descriptions
*/

/*
SPI Interrupt Enable (for SPRF and MODF)  This is the interrupt enable for SPI receive buffer full (SPRF)
and mode fault (MODF) events.
0 Interrupts from SPRF and MODF inhibited (use polling).
1 When SPRF or MODF is 1, request a hardware interrupt.
*/
#define SPIE_  7

/*
SPI System Enable  Disabling the SPI halts any transfer that is in progress, clears data buffers, and initializes
internal state machines. SPRF is cleared and SPTEF is set to indicate the SPI transmit data buffer is empty.
0 SPI system inactive.
1 SPI system enabled.
*/
#define SPE_   6

/*
SPI Transmit Interrupt Enable  This is the interrupt enable bit for SPI transmit buffer empty (SPTEF).
0 Interrupts from SPTEF inhibited (use polling).
1 When SPTEF is 1, hardware interrupt requested.
*/
#define SPTIE_ 5

/*
Master/Slave Mode Select
0 SPI module configured as a slave SPI device.
1 SPI module configured as a master SPI device.
*/
#define MSTR_  4

/*
Clock Polarity  This bit effectively places an inverter in series with the clock signal from a master SPI or to a
slave SPI device. Refer to Section 4.7.1, SPI Clock Formats, for more details.
0 Active-high SPI clock (idles low).
1 Active-low SPI clock (idles high).
*/
#define CPOL_  3

/*
Clock Phase  This bit selects one of two clock formats for different kinds of synchronous serial peripheral
devices. Refer to Section 4.7.1, SPI Clock Formats, for more details.
0 First edge on SPSCK occurs at the middle of the first cycle of an 8-cycle data transfer.
1 First edge on SPSCK occurs at the start of the first cycle of an 8-cycle data transfer.
*/
#define CPHA_  2

/*
Slave Select Output Enable  This bit is used in combination with the mode fault enable (MODFEN) bit in
SPCR2 and the master/slave (MSTR) control bit to determine the function of the SS1 pin as shown in Table 4-4.
*/
#define SSOE_  1

/*
LSB First (Shifter Direction)
0 SPI serial data transfers start with most significant bit.
1 SPI serial data transfers start with least significant bit.
*/
#define LSBFE_ 0

/*
===============================================================================================================*/


/*===============================================================================================================
                                        SPI1C2 Field Descriptions
*/

/*
Master Mode-Fault Function Enable  When the SPI is configured for Slave Mode, this bit has no meaning or
effect. (The SS1 pin is the slave select input.) In Master Mode, this bit determines how the SS1 pin is used (refer
to Table 4-4 for more details).
0 Mode fault function disabled, master SS1 pin reverts to general-purpose I/O not controlled by SPI.
1 Mode fault function enabled, master SS1 pin acts as the mode fault input or the slave select output.
*/
#define  MODFEN_  4

/*
Bidirectional Mode Output Enable  When Bidirectional Mode is enabled by SPI pin control 0 (SPC0) = 1,
BIDIROE determines whether the SPI data output driver is enabled to the single bidirectional SPI I/O pin.
Depending on whether the SPI is configured as a master or a slave, it uses either the MOSI1 (MOMI) or MISO1
(SISO) pin, respectively, as the single SPI data I/O pin. When SPC0 = 0, BIDIROE has no meaning or effect.
0 0utput driver disabled so SPI data I/O pin acts as an input.
1 SPI I/O pin enabled as an output.
*/
#define  BIDIROE_ 3

/*
SPI Stop in Wait Mode
0 SPI clocks continue to operate in Wait Mode.
1 SPI clocks stop when the MCU enters Wait Mode.
*/
#define  SPISWAI_ 1

/*
SPI Pin Control 0  The SPC0 bit chooses Single-wire Bidirectional Mode. If MSTR = 0 (Slave Mode), the SPI
uses the MISO1 (SISO) pin for bidirectional SPI data transfers. If MSTR = 1 (Master Mode), the SPI uses the
MOSI1 (MOMI) pin for bidirectional SPI data transfers. When SPC0 = 1, BIDIROE is used to enable or disable
the output driver for the single bidirectional SPI I/O pin.
0 SPI uses separate pins for data input and data output.
1 SPI configured for single-wire bidirectional operation.
*/
#define  SPC0_    0
/*
===============================================================================================================*/

/*===============================================================================================================
                                        SPI1BR Field Descriptions
*/

/*
SPI Baud Rate Prescale Divisor  This 3-bit field selects one of eight divisors for the SPI baud rate prescaler
as shown in Table 4-7. The input to this prescaler is the bus rate clock (BUSCLK). The output of this prescaler
drives the input of the SPI baud rate divider (see Figure 4-5).

SPPR2:SPPR1:SPPR0   Prescaler Divisor
0:0:0 1
0:0:1 2
0:1:0 3
0:1:1 4
1:0:0 5
1:0:1 6
1:1:0 7
1:1:1 8
*/
#define  SPPR2_   6
#define  SPPR1_   5
#define  SPPR0_   4

/*
SPI Baud Rate Divisor  This 3-bit field selects one of eight divisors for the SPI baud rate divider as shown in
Figure 4-8. The input to this divider comes from the SPI baud rate prescaler (see Figure 4-5). The output of this
divider is the SPI bit rate clock for Master Mode.

SPR2:SPR1:SPR0   Rate Divisor
0:0:0 2
0:0:1 4
0:1:0 8
0:1:1 16
1:0:0 32
1:0:1 64
1:1:0 128
1:1:1 256
*/
#define  SPR2_    2
#define  SPR1_    1
#define  SPR0_    0

/*
===============================================================================================================*/

/*===============================================================================================================
                                        SPI1S Field Descriptions
*/

/*
SPI Read Buffer Full Flag  SPRF is set at the completion of an SPI transfer to indicate that received data may
be read from the SPI data register (SPI1D). SPRF is cleared by reading SPRF while it is set, then reading the
SPI data register.
0 No data available in the receive data buffer.
1 Data available in the receive data buffer.
*/
#define  SPRF_   7

/*
SPI Transmit Buffer Empty Flag  This bit is set when there is room in the transmit data buffer. It is cleared by
reading SPI1S with SPTEF set, followed by writing a data value to the transmit buffer at SPI1D. SPI1S must be
read with SPTEF = 1 before writing data to SPI1D or the SPI1D write will be ignored. SPTEF generates an
SPTEF CPU interrupt request if the SPTIE bit in the SPI1C1 is also set. SPTEF is automatically set when a data
byte transfers from the transmit buffer into the transmit shift register. For an idle SPI (no data in the transmit buffer
or the shift register and no transfer in progress), data written to SPI1D is transferred to the shifter almost
immediately so SPTEF is set within two bus cycles allowing a second 8-bit data value to be queued into the
transmit buffer. After completion of the transfer of the value in the shift register, the queued value from the
transmit buffer will automatically move to the shifter and SPTEF will be set to indicate there is room for new data
in the transmit buffer. If no new data is waiting in the transmit buffer, SPTEF simply remains set and no data
moves from the buffer to the shifter.
0 SPI transmit buffer not empty.
1 SPI transmit buffer empty.
*/
#define  SPTEF_   5

/*
Master Mode Fault Flag  MODF is set if the SPI is configured as a master and the slave select input goes
low, indicating some other SPI device is also configured as a master. The SS1 pin acts as a mode fault error input
only when MSTR = 1, MODFEN = 1, and SSOE = 0; otherwise, MODF will never be set. MODF is cleared by
reading MODF while it is 1, then writing to SPI control register 1 (SPI1C1).
0 No mode fault error.
1 Mode fault error detected.
*/
#define  MODF_   4

/*
===============================================================================================================*/

/*-------------------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------------------*/
void SPI_Init(void)
{
  SPI1C1 = BIT(SPE_) + BIT(MSTR_);
  SPI1C2 = 0x00;
  SPI1BR = BIT(SPPR1_); //     SPI  3
}


/*-------------------------------------------------------------------------------------------------------------
    RF    u8Addr  u16Content

  Parameters : u8Addr - SPI address
  u16Content - Data to send

  Return : None
-------------------------------------------------------------------------------------------------------------*/
void SPI_Write(INT8U u8Addr, INT16U u16Content)
{
  volatile INT8U u8TempValue;

  u8TempValue = SPI1S;          //   
  u8TempValue = SPI1D;
//  Phy_Disable_interrupts();
  MODEM_CE_LOW;
  SPI1D = u8Addr & 0x3F;        //  6-  
  while( _SPI1S.Bits.SPRF==0 ); //  
  u8TempValue = SPI1D;          //    SPRF     
  SPI1D = u16Content >> 8;      //   
  while( _SPI1S.Bits.SPRF==0 );
  u8TempValue = SPI1D;
  SPI1D = u16Content & 0x00FF;
  while( _SPI1S.Bits.SPRF==0 );
  u8TempValue = SPI1D;
  MODEM_CE_HIGH;
//  Phy_Restore_interrupts(); // Restore MC13192 interrupt status
}


/*-------------------------------------------------------------------------------------------------------------
        u8Addr


  Parameters : u8Addr - SPI address

  Return : u16Data -  u16Data[0] is the MSB, u16Data[1] is the LSB
-------------------------------------------------------------------------------------------------------------*/
INT16U SPI_Read(INT8U u8Addr)
{
  volatile INT8U u8TempValue;
  INT16U  u16Data;

  u8TempValue = SPI1S;           //   
  u8TempValue = SPI1D;
//  Phy_Disable_interrupts();
  MODEM_CE_LOW;
  SPI1D = (u8Addr & 0x3f) | 0x80;//        
  while( _SPI1S.Bits.SPRF==0 );
  u8TempValue = SPI1D;           //    SPRF     
  SPI1D = u8Addr ;               //       
  while( _SPI1S.Bits.SPRF==0 );
  u16Data = SPI1D << 8;
  SPI1D = u8Addr;                //       
  while( _SPI1S.Bits.SPRF==0 );
  u16Data |= SPI1D;
  MODEM_CE_HIGH;                 // Disables MC13192 SPI
//  Phy_Restore_interrupts();    // Restore MC13192 interrupt status
  return u16Data;
}

/*-------------------------------------------------------------------------------------------------------------
 * SPI_SecondRead : Read the second word from SPI
 *
 * Parameters : u8Addr - SPI address
 *
 * Return : u16Data -  u16Data[0] is the MSB, u16Data[1] is the LSB
-------------------------------------------------------------------------------------------------------------*/
INT16U SPI_ReadSecondWord(INT8U addr)
{
  volatile INT8U tmp;
  INT16U  data;

  tmp = SPI1S;            //   
  tmp = SPI1D;
  MODEM_CE_LOW;
  SPI1D = (addr & 0x3f) | 0x80; //        
  while( _SPI1S.Bits.SPRF==0 );
  tmp = SPI1D;            // Clear receive data register. SPI entirely ready for read or write
  SPI1D = addr ;                // Dummy write. Receive register of SPI will contain MSB
  while( _SPI1S.Bits.SPRF==0 );
  tmp = SPI1D;
  SPI1D = addr;                // Dummy write. Waiting until after reading received data insures no overrun
  while( _SPI1S.Bits.SPRF==0 );
  tmp = SPI1D;
  SPI1D = addr;                // Dummy write. Waiting until after reading received data insures no overrun
  while( _SPI1S.Bits.SPRF==0 );
  data = SPI1D << 8;
  SPI1D = addr;                // Dummy write. Waiting until after reading received data insures no overrun
  while( _SPI1S.Bits.SPRF==0 );
  data |= SPI1D;
  MODEM_CE_HIGH;
  return data;
}

