#include "includes.h"

/*===============================================================================================================
                                        TPM1SC Register Field Descriptions
*/

/*
Timer Overflow Flag  This flag is set when the TPM counter changes to $0000 after reaching the modulo
value programmed in the TPM counter modulo registers. When the TPM is configured for CPWM, TOF is set
after the counter has reached the value in the modulo register, at the transition to the next lower count value.
Clear TOF by reading the TPM status and control register when TOF is set and then writing a 0 to TOF. If another
TPM overflow occurs before the clearing sequence is complete, the sequence is reset so TOF would remain set
after the clear sequence was completed for the earlier TOF. Reset clears TOF. Writing a 1 to TOF has no effect.
0 TPM counter has not reached modulo value or overflow
1 TPM counter has overflowed
*/
#define TOF_    7

/*
Timer Overflow Interrupt Enable  This read/write bit enables TPM overflow interrupts. If TOIE is set, an
interrupt is generated when TOF equals 1. Reset clears TOIE.
0 TOF interrupts inhibited (use software polling)
1 TOF interrupts enabled
*/
#define TOIE_   6

/*
Center-Aligned PWM Select  This read/write bit selects CPWM operating mode. Reset clears this bit so the
TPM operates in Up-counting Mode for input capture, output compare, and edge-aligned PWM functions. Setting
CPWMS reconfigures the TPM to operate in Up-/down-counting Mode for CPWM functions. Reset clears
CPWMS.
0 All TPM1 channels operate as input capture, output compare, or Edge-aligned PWM Mode as selected by the
MSnB:MSnA control bits in each channels status and control register
1 All TPM1 channels operate in Center-aligned PWM Mode
*/
#define CPWMS_  5

/*
Clock Source Select  As shown in Table 17-2, this 2-bit field is used to disable the TPM system or select one
of three clock sources to drive the counter prescaler. The external source and the XCLK are synchronized to the
bus clock by an on-chip synchronization circuit.

"CLKS1:CLKS0"    "TPM Clock Source to Prescaler Input"
0:0              No clock selected (TPM disabled)
0:1              Bus rate clock (BUSCLK)
1:0              Fixed system clock (XCLK)
1:1              External source (TPM1 Ext Clk)1,2
*/
#define CLKS1_  4
#define CLKS0_  3

/*
Prescale Divisor Select  This 3-bit field selects one of eight divisors for the TPM clock input as shown in
Table 17-3. This prescaler is located after any clock source synchronization or clock source selection, so it affects
whatever clock source is selected to drive the TPM system.
PS2:PS1:PS0   TPM Clock Source Divided-By
0:0:0         1
0:0:1         2
0:1:0         4
0:1:1         8
1:0:0         16
1:0:1         32
1:1:0         64
1:1:1         128
*/
#define PS2_    2
#define PS1_    1
#define PS0_    0

/*
===============================================================================================================*/


/*===============================================================================================================
                                        TPM1CnSC Register Field Descriptions (n = 0..3)
*/

/*
Channel n Flag  When channel n is configured for input capture, this flag bit is set when an active edge occurs
on the channel n pin. When channel n is an output compare or edge-aligned PWM channel, CHnF is set when
the value in the TPM counter registers matches the value in the TPM channel n value registers. This flag is
seldom used with center-aligned PWMs because it is set every time the counter matches the channel value
register, which correspond to both edges of the active duty cycle period.
A corresponding interrupt is requested when CHnF is set and interrupts are enabled (CHnIE = 1). Clear CHnF
by reading TPM1CnSC while CHnF is set and then writing a 0 to CHnF. If another interrupt request occurs before
the clearing sequence is complete, the sequence is reset so CHnF would remain set after the clear sequence
was completed for the earlier CHnF. This is done so a CHnF interrupt request cannot be lost by clearing a
previous CHnF. Reset clears CHnF. Writing a 1 to CHnF has no effect.
0 No input capture or output compare event occurred on channel n
1 Input capture or output compare event occurred on channel n
*/
#define CHF_  7

/*
Channel n Interrupt Enable  This read/write bit enables interrupts from channel n. Reset clears CHnIE.
0 Channel n interrupt requests disabled (use software polling)
1 Channel n interrupt requests enabled
*/
#define CHIE_  6

/*
Mode Select B for TPM Channel n  When CPWMS = 0, MSB = 1 configures TPM channel n for
edge-aligned PWM Mode. For a summary of Channel Mode and setup controls, refer to Table 17-5.
*/
#define MSB_  5

/*
Mode Select A for TPM Channel n  When CPWMS = 0 and MSB = 0, MSnA configures TPM channel n for
Input Capture Mode or Output Compare Mode. Refer to Table 17-5 for a summary of Channel Mode and setup
controls.
*/
#define MSA_  4

/*
Edge/Level Select Bits  Depending on the operating mode for the timer channel as set by
CPWMS:MSnB:MSnA and shown in Table 17-5, these bits select the polarity of the input edge that triggers an
input capture event, select the level that will be driven in response to an output compare match, or select the
polarity of the PWM output.
Setting ELSnB:ELSnA to 0:0 configures the related timer pin as a general-purpose I/O pin unrelated to any timer
channel functions. This function is typically used to temporarily disable an input capture channel or to make the
timer pin available as a general-purpose I/O pin when the associated timer channel is set up as a software timer
that does not require the use of a pin. This is also the setting required for channel 0 when the TPM1CH0 pin is
used as an external clock input.
*/
#define ELS1_  3
#define ELS0_  2

/*
===============================================================================================================*/


/* Globals */
INT32U cnt_0_8_ms = 0;
INT16U gu16timerEvent[3];
extern INT16U gu16Events;

static INT32U bus_freq; //    
static INT8U  pttn        = 0;
static INT8U  prev_pttn   = 0;
static INT8U  sw_poll_cnt = SW_POLL_CNT;
static INT8U  sw_poll_timeout;
static INT8U  buttn_state = 0;

/*-------------------------------------------------------------------------------------------------------------
        
-------------------------------------------------------------------------------------------------------------*/
void  Timers_set_bus_freq(INT32U freq)
{
  bus_freq = freq;
}

/*-------------------------------------------------------------------------------------------------------------
      1
-------------------------------------------------------------------------------------------------------------*/
void Timer1_init(void)
{
  TPM1MOD = 0;
  //       = 128
  //     = 156250 Hz (period = 6.4 us)    = 20 MHz
  TPM1SC = BIT(CLKS0_) + BIT(PS0_) +  BIT(PS1_) + BIT(PS2_);

  //  0        
  TPM1C0SC = BIT(MSA_); //       

  TPM1C1V  = INT_PRESCALE;
  //  1        
  TPM1C1SC = BIT(CHIE_) + BIT(MSA_); //       
}

/*-------------------------------------------------------------------------------------------------------------
          1
-------------------------------------------------------------------------------------------------------------*/
void Timer1_delay(INT16U t)
{
  if (t==0) t=1;
  TPM1C0V = TPM1CNT + t;
  TPM1C0SC = TPM1C0SC & (~BIT(CHF_)); //   
  while ((TPM1C0SC & BIT(CHF_))==0);   //   
}


/*-------------------------------------------------------------------------------------------------------------
      1  1
      1250 
-------------------------------------------------------------------------------------------------------------*/
interrupt void IRQ_TPM1C1(void)
{


  TPM1C1V = TPM1CNT + INT_PRESCALE;



  if (sw_poll_cnt==0)
  {
    LED4 = 1;
    sw_poll_cnt = SW_POLL_CNT;

    //  
    pttn = 0;
    //     
    pttn = ((PTAD & (BIT(5) | BIT(4) | BIT(3) | BIT(2))) ^ (BIT(5) | BIT(4) | BIT(3) | BIT(2))) >> 2;

    if ( pttn==0 )
    {
      if ( prev_pttn!=0 )
      {
        sw_poll_timeout = BUTTN_FIRST_DELAY/BUTTN_CYCLE_PER;
      }
    }
    else
    {
      if ( (pttn & ~prev_pttn)!=0 ) //     
      {
        //     
        sw_poll_timeout = BUTTN_FIRST_DELAY/BUTTN_CYCLE_PER;
        buttn_state = pttn;
      }
      else
      {
        if ( sw_poll_timeout==0 )
        {
          //      
          sw_poll_timeout = BUTTN_NEXT_DELAY/BUTTN_CYCLE_PER;
          buttn_state = pttn;
        }
        else
          sw_poll_timeout -=1;

        if ( pttn!=prev_pttn )
        {
          sw_poll_timeout = BUTTN_FIRST_DELAY/BUTTN_CYCLE_PER;
        }
      }
    }
    prev_pttn = pttn; //     
    LED4 = 0;
  }
  sw_poll_cnt--;

  cnt_0_8_ms++;

  TPM1C1SC &= ~BIT(CHF_);          /* clear the interrupt */

}

/**************************************************************
*    Function:      Delay
*    Parameters:    16-bit Delay value
*    Return:        none
*
*    Simple blocking Delay loop using HW timer.
**************************************************************/

void Delay_by_ticks(INT16U cnt)
{
  INT32U v;

  v = cnt_0_8_ms + cnt;
  while ( cnt_0_8_ms != v );
}


/*
void Wait1ms(void)
{
  asm {
    PSHX
    PSHH
    LDHX #CONST
    loop:
    AIX #-1  ;2 cycles
    CPHX #0  ;3 cycles
    BNE loop ;3 cycles
    PULH
    PULX
  }
}
*/


/*-------------------------------------------------------------------------------------------------------------
      
-------------------------------------------------------------------------------------------------------------*/
INT8U Get_SW_state(INT8U sw_n)
{
  if ((buttn_state & BIT(sw_n))!=0)
  {
    buttn_state &= ~BIT(sw_n);
    return TRUE;
  }
  return FALSE;
}
